//日射比例灌水装置制御プログラム
//2018/07/13 H.Kurosaki
//注意：LCDとTSL2561のライブラリが必要です
#include "FaBoLCDmini_AQM0802A.h"
#include <SparkFunTSL2561.h>
#include <Wire.h>

//LCD
FaBoLCDmini_AQM0802A lcd;

//TSL2561
char TSL2561_sensSts;
SFE_TSL2561 light;
unsigned int TSL2561_Waitms;
#define TSL2561_GAIN 0

//デジタルピン番号の定義
#define LED_RED		9	//ピン番号 赤LEDと灌水停止信号
#define LED_YELLOW	5	//ピン番号 黄LEDと灌水開始信号
#define LED_GREEN	7	//ピン番号 緑LED(動作確認用)
#define SUNRISE_MODEJP	A3	//ピン番号 日の出同時灌水モード設定ジャンパー(負論理)
//アナログピン番号
#define ANALOG_VOLUME_FREQUENCY	0
#define ANALOG_VOLUME_TIME	1
#define ANALOG_3PSW		2

//切り替えスイッチ用の定義
//3pスイッチはA2に接続されていて上に倒すと5V、下に倒すと3.3V、中央でGNDに接続される
#define AIN_LOWLIMIT	200	//スイッチ下側の判定しきい値
#define AIN_HIGHLIMIT	800	//スイッチ上側の判定しきい値
#define SWMODE_MANON	1	//手動ON
#define SWMODE_MANOFF	0	//手動OFF
#define SWMODE_AUTO	2	//自動モード

//積算日射カウント用
//日射量の単位は瞬間値にkW/m2、積算値だとMJ/m2がよく使われますが、
//このプログラムは内部での積算値にkW・h/m2を使用し表示のみMJに換算しています
static double radiation_count=0;	//積算日射量(kW・h/m2)
static double radiation_limit_WH=0;	//灌水を開始する積算日射量(kW・h/m2)
static double radiation_limit_MJH=0;	//灌水を開始する積算日射量(MJ/m2)
static double radiation_limit_avrtimes=0;//平均的な日射量の日の灌水回数予測値(回)

//lux→kW/m2変換係数（減光フィルター込み）センサの作り方によって若干変わる
#define SENSOR_TO_RADIATION 0.0001355 

//日本の平均的な1日の積算日射量(kW・h/m2)
#define MAX_INTRADIATION 3.5

//これ以下の日射量(kW/m2)では潅水しない(積算はする)
#define LOWLIGHT_LIMIT 0.03

//暗期がこれ（分）以上続いたら夜になったと見なす(早朝灌水機能設定時)
#define MIN_NIGHT_DURATION 180

//潅水時間カウント用変数
int irrigation_seccount=0;//灌水時間カウンタ(sec)
int irrigation_secmax=0;//ユーザーが設定した灌水時間(sec)
//暗期カウント用変数
unsigned int night_mincount=0;//夜時間カウンタ(min)
//潅水中フラグ用変数
bool flag_irrigation_start=false;//灌水実施中でtrueになる

//起動時の初期化に１回だけ実行される関数
void setup() {
 pinMode(LED_RED,OUTPUT);
 pinMode(LED_YELLOW,OUTPUT);
 pinMode(LED_GREEN,OUTPUT);
 pinMode(SUNRISE_MODEJP,INPUT_PULLUP);//早朝灌水設定ジャンパー………(A)
 LCDStart();
}

//無限にループする関数
void loop() {

//前回の実行から0.1秒経っていない場合は何もしない…………………………(B)
static unsigned long lastsec=0;
unsigned long nowsec=millis()/100;
if(nowsec==lastsec){return;}
 lastsec=nowsec;
 //これ以降に0.1秒に１回実行される処理を書く
 
 //ボリューム設定値の読み取り……………………………………………………(C)
 //積算日射量の設定
 int freqx10=ReadVolumeFrequency()/10;//ボリュームの値を0-102の範囲に収める(1→0.1MJ)
 if(freqx10<=0){freqx10=1;}//最小値が0にならないようにする
 //潅水を行う積算日射量のしきい値(MJ/m2)
 radiation_limit_MJH=freqx10/10.0;//さらに1/10して1～10.2MJ/m2の範囲とする
 radiation_limit_WH=radiation_limit_MJH/3.6;//MJ/m2→kWH/m2換算
 radiation_limit_avrtimes=MAX_INTRADIATION/radiation_limit_WH;//平均的な日の潅水回数計算

 //灌水時間の設定
 int timex10=ReadVolumeTime()/10;//ボリュームの値を0-102の範囲に収める(1→0.1分)
 if(timex10<=0){timex10=1;}//最小値が0にならないようにする
 irrigation_secmax=timex10*6;//秒になおす(設定は0.1分=6秒単位)

 //照度計測
 long lux=TSL2561_readSensor();
 if(TSL2561_sensSts)//センサにアクセスできない場合true
	 {
	 //初回起動時またはセンサ不調時に初期化処理を行う
	 light.begin();
	 light.setPowerDown();
	 light.setTiming(TSL2561_GAIN,0,TSL2561_Waitms);
	 light.setPowerUp();
	 TSL2561_sensSts=0;
	 }
 //単位を換算…………………………………………………………………………(D)
 double radiation=(double)lux*SENSOR_TO_RADIATION;//lux->日射量(kW/m2)換算
  
 //手動自動切り替えスイッチを読む………………………………………………(E)
 int nowsw=ReadSW();
 //LCDを消去
 lcd.clear();
 //スイッチが入っている間はずっと実行する処理
 if(nowsw==SWMODE_AUTO)
	 {
	//緑LEDを点滅
	 digitalWrite(LED_GREEN,!digitalRead(LED_GREEN));
	 lcd.print("AUTO ");//自動モードの表示
	 AutoIrrigation(radiation);//灌水制御の実行
	 }
 else if(nowsw==SWMODE_MANON)
	 {
	 digitalWrite(LED_GREEN,HIGH);
	 lcd.print("MANON ");//手動ON表示0
   flag_irrigation_start=false;//自動で灌水中の場合止める
	 }
 else if(nowsw==SWMODE_MANOFF)
	 {
	digitalWrite(LED_GREEN,LOW);
	 lcd.print("MANOFF ");//手動OFF表示
   flag_irrigation_start=false;//自動で灌水中の場合止める
	 }
 
	//自動モードで灌水中の場合は表示を変更する
	 if(flag_irrigation_start)
	 {
	 lcd.print("OPEN:");
	//残時間を表示する
	 lcd.print(irrigation_secmax-irrigation_seccount);
	 }
 else if(nowsw==SWMODE_AUTO)
	 {
	//自動モード

	//早朝灌水が設定されていて夜の場合
	 if(radiation<LOWLIGHT_LIMIT &&digitalRead(SUNRISE_MODEJP)==LOW){lcd.print("WAITING SUN");}
	//通常の自動モードで夜の場合
	 else if(radiation<LOWLIGHT_LIMIT){lcd.print("LOW LIGHT");}
	//自動モードで日中の場合
	 else
		 {
		 lcd.print(radiation_count*3.6,2);//積算日射量の表示(kWH/m2→MJ/m2に換算して表示する)
		 lcd.print("MJ ");
		 lcd.print((radiation_count/radiation_limit_WH)*100,0);//日射の蓄積具合を%で表示
		 lcd.print("%");
		 }
	 }
 else
	 {
 //手動モードでは簡易日射計になる(日射量の瞬時値はkW/m2)
 lcd.print(radiation,2);
 lcd.print("kW/m2");
 }
 //LCD下半分の表示
 lcd.command(0x40+0x80);//改行
 //以下はボリュームを操作した時だけしばらく表示が変わる処理
 static int lastVolumeFrequency=-1;
 static int lastVolumeTime=-1;
 static int dispCountFrequency=0;
 static int dispCountTime=0;
 if(ReadVolumeFrequency()!=lastVolumeFrequency)
	 {
	 dispCountFrequency=100;
	 dispCountTime=0;
	 }
 lastVolumeFrequency=ReadVolumeFrequency();
 if(ReadVolumeTime()!=lastVolumeTime)
	 {
	 dispCountFrequency=0;
	 dispCountTime=100;
	 }
 lastVolumeTime=ReadVolumeTime();
 //VolumeFrequencyを動かしたとき、しばらく積算日射量と潅水回数の詳細を表示
 if(dispCountFrequency>0)
	 {
	 dispCountFrequency--;
	 lcd.print(radiation_limit_MJH,1);
	 lcd.print("MJ/m2 FRQ");
	 lcd.print(radiation_limit_avrtimes,1);//平均的な日射量から推定した1日の潅水回数
	 }
 //VolumeTimeを動かしたとき、しばらく潅水時間の詳細を表示
 else if(dispCountTime>0)
	 {
	 dispCountTime--;
	 lcd.print((float)timex10/10,1);
	 lcd.print("min");
	 lcd.print(" "); 
	 lcd.print(timex10*60/10);
	 lcd.print("sec");
	 }
 //普段は積算日射量と潅水時間を簡易表示
 else
	 {
	 lcd.print(radiation_limit_MJH,1);
	 lcd.print("MJ/");
	 lcd.print(" ");
	 lcd.print((float)timex10/10,1);
	 lcd.print("min"); 
	 }
 //---------------------------------------
 //以下はスイッチが切り替わった瞬間だけ実行する処理
 static int lastsw=-1;//初回はありえない値で初期化しておく
 if(lastsw!=nowsw&&nowsw==SWMODE_AUTO)
	 {
	 //自動潅水
	 //切り替わった瞬間にバルブが開いていた場合は止める
	 ValveClose();
	 }
 else if(lastsw!=nowsw&&nowsw==SWMODE_MANON)
	 {
	 //手動灌水
	 ValveOpen();
	 }
 else if(lastsw!=nowsw&&nowsw==SWMODE_MANOFF)
	 {
	 //手動停止
	 ValveClose();
	 }
 lastsw=nowsw;
 //---------------------------------------
}
//-----------------自動潅水の制御
bool AutoIrrigation(double radiation)
{
static unsigned long lastsec=0;
unsigned long nowsec=millis()/1000;
//前回実行時から１秒経っていない場合は何もしない
if(nowsec==lastsec){return flag_irrigation_start;}
 lastsec=nowsec;
 
 if(!flag_irrigation_start&&nowsec%60==0)
	 {
	 //ここには灌水中以外で１分に１回実行される処理を書く…………(F)

	 radiation_count+=radiation/60;//積算が1分間隔なので60で割って1時間単位の積算値に換算
	//オーバーフロー防止
	 if(radiation_count>radiation_limit_WH)
		 {radiation_count=radiation_limit_WH;}
	 
	 //暗期の持続時間を最大1440分まで積算する
	 if(radiation<=LOWLIGHT_LIMIT)
		 {
		 night_mincount++;
		//オーバーフロー防止
		 if(night_mincount>1440){night_mincount=1440;}
		 }
	//早朝灌水モードがOFFで日が出たとき
	if(radiation>LOWLIGHT_LIMIT && digitalRead(SUNRISE_MODEJP))
		{
		night_mincount=0;
		}

	 }
	//以下は毎秒実行される処理………………………………………………(G)
	 //潅水中の場合
if(flag_irrigation_start)
	{ 
		 irrigation_seccount++;
		 //潅水停止
		 if(irrigation_seccount>=irrigation_secmax)
			 {
			 flag_irrigation_start=false;
			 ValveClose();
			 } 
	 } 
 //早朝灌水モード
 //SUNRISE_MODEJPのジャンパーが設定されており、暗期がMIN_NIGHT_DURATION以上持続したあと
 //十分に明るくなった場合、日の出と見なして強制灌水を行う
 else if(digitalRead(SUNRISE_MODEJP)==LOW && radiation>LOWLIGHT_LIMIT && night_mincount>=MIN_NIGHT_DURATION)
	 {
	//カウンタの初期化
	 night_mincount=0;
	 radiation_count=0;
	 irrigation_seccount=0;
	//自動潅水の開始
	 flag_irrigation_start=true;
	 ValveOpen();
	 }
 //通常潅水
 //通常モード時、日射量がLOWLIGHT_LIMIT以上の時だけ潅水する
 else if(radiation_count>=radiation_limit_WH && radiation>LOWLIGHT_LIMIT)
	 {
	//カウンタの初期化
	 radiation_count=0;
	 irrigation_seccount=0;
	 //自動潅水の開始
	 flag_irrigation_start=true;
	 ValveOpen();
	 }
 
return flag_irrigation_start;
}
//--------------電磁弁の制御………………………………………………………(H)
void ValveOpen()
{
 //ラッチバルブ 
 digitalWrite(LED_YELLOW,HIGH);
 digitalWrite(LED_RED,LOW); 
 delay(1000);//1秒通電してから止める
 digitalWrite(LED_YELLOW,LOW);
 digitalWrite(LED_RED,LOW);
}
void ValveClose()
{
 //ラッチバルブ 
 digitalWrite(LED_YELLOW,LOW);
 digitalWrite(LED_RED,HIGH);
 delay(1000);//1秒通電してから止める
 digitalWrite(LED_YELLOW,LOW);
 digitalWrite(LED_RED,LOW);
 
}
void ValveNeutral()
{
 digitalWrite(LED_YELLOW,LOW);
 digitalWrite(LED_RED,LOW);
 }
//----------------------------
//アナログ入力を1023段階で出力する、ノイズ消し
int ReadVolumeFrequency()
{
int a=1023-analogRead(ANALOG_VOLUME_FREQUENCY);
static int pr;
//AD値が±2以上変化したときだけ戻り値を変更
if (abs(pr-a)>2){pr=a;}
return pr;
}
//----------------------------
//アナログ入力を1023段階で出力する、ノイズ消し
int ReadVolumeTime()
{
int a=1023-analogRead(ANALOG_VOLUME_TIME);
static int pr;
//AD値が±2以上変化したときだけ戻り値を変更
if (abs(pr-a)>2){pr=a;} 
return pr; 
}
//--------------切り替えスイッチを読み出す
int ReadSW()
{
int sw=analogRead(ANALOG_3PSW);
if(sw<AIN_LOWLIMIT){return SWMODE_MANOFF;}//中央
else if(sw<AIN_HIGHLIMIT){return SWMODE_MANON;}//下側
//上側
return SWMODE_AUTO;
}
//-------------LCDの初期化
void LCDStart()
{
 lcd.begin();
 lcd.init();
//for I2C voltage 5V
lcd.command(0x38);delay(1);
lcd.command(0x39);delay(1);
lcd.command(0x14);delay(1);
lcd.command(0x73);delay(1);
lcd.command(0x51);delay(2);
lcd.command(0x6c);delay(300);
lcd.command(0x38);delay(1);
lcd.command(0x01);delay(2);
lcd.command(0x0c);
} 
//------------TSL2561を読み出す
long TSL2561_readSensor(void)
{
//Light
unsigned int data0, data1;
double lux;
if(!light.getData(data0,data1)){TSL2561_sensSts=1;return -1;}
if(!light.getLux(TSL2561_GAIN,TSL2561_Waitms,data0,data1,lux)){TSL2561_sensSts=1;return -1;}
 return lux;
}
