//SEEDLING BOX CONTROLLER, M.Urushidani
#define HZ60  //if 50Hz then comment out this line!(//)

#include <Wire.h> 
#include <FaBoLCDmini_AQM0802A.h>
// initialize the library
FaBoLCDmini_AQM0802A lcd;
//I/O
int clk = 8;      //rotary encoder pin1
int data = 9;     //rotary encoder pin2
//Thermistor Murata
int   a_val = 0;
double vcc  = 5.0;
double vol  = 0.0;
double temp = 0.0;
//Variables
float set_temperature = 0;   //Default temperature setpoint
float temperature_read = 0.0;
float PID_error = 0;
#ifdef HZ60
float Phase_max = 7680;
#else
float Phase_max = 9200;
#endif
float previous_error = 0;
float elapsedTime, Time, timePrev;
float PID_value = 0;
int button_pressed = 0;
int menu_activated=0;
float last_set_temperature = 0;
float CONT_phase = 0;
//Vraiables for rotary encoder state detection
int clk_State;
int Last_State;  
bool dt_State;  
//PID constants
int kp = 89;   int ki = 29;   int kd = 79;
int PID_p = 0;    int PID_i = 0;    int PID_d = 0;
float last_kp = 0;
float last_ki = 0;
float last_kd = 0;
int PID_values_fixed =0;
#define triacPulse 5
#define SW 4
#define aconLed 12
#define lampSSR 6
#define fanSSR 7 
int val;

void setup() {
  // set up the LCD:
  lcd.begin();
  lcd.command(0x38);  //initialize:size:8x2line,font5*7
  lcd.command(0x39);  //expand command, IS=1
  lcd.command(0x14);  //internal osc. freq.
  lcd.command(0x73);  //contrast
  lcd.command(0x51);  //vcc,contrast
  lcd.command(0x6c);  //follower on
  lcd.command(0x38);  //end of expand command, IS=0
  lcd.command(0x01);  //clear display
  lcd.command(0x0c);  //display on
  
  Time = millis();
  
  Last_State = (PINB & B00000001);      //Detect first state of the encoder

  PCICR |= (1 << PCIE0);    //enable PCMSK0 scan                                                 
  PCMSK0 |= (1 << PCINT0);  //Set pin D8 trigger an interrupt on state change. 
  PCMSK0 |= (1 << PCINT1);  //Set pin D9 trigger an interrupt on state change. 
  PCMSK0 |= (1 << PCINT3);  //Set pin D11 trigger an interrupt on state change.   
                           
  pinMode(11,INPUT);
  pinMode(9,INPUT);
  pinMode(8,INPUT);

  analogReference(DEFAULT);

  //triac control
  pinMode(2, INPUT);
  digitalWrite(2, HIGH); // pull up
  pinMode(triacPulse, OUTPUT);
  pinMode(SW, INPUT);
  digitalWrite(SW, HIGH);
  pinMode(aconLed, OUTPUT);
  digitalWrite(aconLed, LOW);

  //SSR control
  pinMode(lampSSR,OUTPUT);
  digitalWrite(lampSSR,LOW);
  pinMode(fanSSR,OUTPUT);
  digitalWrite(fanSSR,LOW);
}

void loop() {
//triac control
// check for SW closed
  if (!digitalRead(SW))   {
    // enable power
    attachInterrupt(0, acon, FALLING); 
    // HV indicator on
    digitalWrite(aconLed, HIGH);
  }  // end if
  else if (digitalRead(SW)) { 
    detachInterrupt(0); // disable power
    // HV indicator off
    digitalWrite(aconLed, LOW);
  }  // else

// Lamp and Fan control
int ad1val = analogRead(1);
float lx = ((long)ad1val*5000/1024)/(290/100);
if (lx > 500) {
  digitalWrite(lampSSR, LOW);
  digitalWrite(fanSSR, HIGH);
}
if (lx < 50) {
  digitalWrite(lampSSR, HIGH);
  digitalWrite(fanSSR, LOW);
}

//PID control
if(menu_activated==0)
{
  // First we read the real value of temperature
  temperature_read = readThermistor();
  //Next we calculate the error between the setpoint and the real value
  PID_error = set_temperature - temperature_read + 1.5;
  //Calculate the P value
  PID_p = 0.3*kp * PID_error;
  //Calculate the I value in a range on +-3
  PID_i = 0.3*PID_i + 30*(ki * PID_error);
  
  //For derivative we need real time to calculate speed change rate
  timePrev = Time;    // the previous time is stored before the actual time read
  Time = millis();    // actual time read
  elapsedTime = (Time - timePrev) / 1000; 
  //Now we can calculate the D calue
  PID_d = 0.3*kd*((PID_error - previous_error)/elapsedTime);
  //Final total PID value is the sum of P + I + D
  PID_value = PID_p + PID_i + PID_d;

  //We define Phase range between 0 and 7680(60Hz)/9200(50Hz)
  if(PID_value < 0)
  {    PID_value = 0;    }
  if(PID_value > Phase_max)
  {    PID_value = Phase_max;  }
  //Now we can write the Trigger phase signal to the TRIAC on digital pin D5
  //Since we activate the TRIAC with a 0 deg to the gate of TRIAC, we write Phase_max-PID value (inverted)
  CONT_phase = (Phase_max-PID_value);
  previous_error = PID_error;     //Remember to store the previous error for next loop.

  delay(250); //Refresh rate + delay of LCD print
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("S:");
  lcd.setCursor(2,0);
  lcd.print(set_temperature,1);
  lcd.setCursor(6,0);
  lcd.write(0b11011111);
  lcd.print("C");
  lcd.setCursor(0,1);
  lcd.print("R:");
  lcd.setCursor(2,1);
  lcd.print(temperature_read,1);
  lcd.setCursor(6,1);
  lcd.write(0b11011111);
  lcd.print("C");
}//end of menu 0 (PID control)

//First page of menu (temp setpoint)
if(menu_activated == 1)
{
  CONT_phase = Phase_max;
  if(set_temperature != last_set_temperature)
  {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Set temp");   
  lcd.setCursor(0,1);
  lcd.print(set_temperature);  
  }
  last_set_temperature = set_temperature;
}//end of menu 1

//Second page of menu (P set)
if(menu_activated == 2)
{
   if(kp != last_kp)
  {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Set Pval");  
  lcd.setCursor(0,1);
  lcd.print(kp);  
  }
  last_kp = kp;
}//end of menu 2

//Third page of menu (I set)
if(menu_activated == 3)
{
  if(ki != last_ki)
  {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Set Ival");
  lcd.setCursor(0,1);
  lcd.print(ki);  
  }
  last_ki = ki;
}//end of menu 3

//Forth page of menu (D set)
if(menu_activated == 4)
{
  if(kd != last_kd)
  {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Set Dval");
  lcd.setCursor(0,1);
  lcd.print(kd);  
  }
  last_kd = kd;
}//end of menu 4
  
}//Loop end

//The function that reads the temperature from Thermistor
double readThermistor() {
  a_val   = analogRead(0);
  vol     = ((double)a_val/1023) * vcc; 
  temp    = -0.30779 * pow(vol,5) + 4.1545 * pow(vol,4) - 23.272 * pow(vol,3) + 68.015 * pow(vol,2) - 126.35 * vol + 160.42;
  return temp;
}

//The interruption vector for push button and rotary encoder
ISR(PCINT0_vect){
if(menu_activated==1)
   {
  clk_State =   (PINB & B00000001); //pin 8 state? Is it HIGH? 
  dt_State  =   (PINB & B00000010); 
  if (clk_State != Last_State){     
     // If the data state is different to the clock state, that means the encoder is rotating clockwise
     if (dt_State != clk_State) { 
       set_temperature = set_temperature+0.5 ;
     }
     else {
       set_temperature = set_temperature-0.5;
     } 
  }
  Last_State = clk_State; // Updates the previous state of the clock with the current state
} 

if(menu_activated==2)
   {
  clk_State =   (PINB & B00000001); //pin 8 state? 
  dt_State  =   (PINB & B00000010); 
  if (clk_State != Last_State){     
     // If the data state is different to the clock state, that means the encoder is rotating clockwise
     if (dt_State != clk_State) { 
       kp = kp+1 ;
     }
     else {
       kp = kp-1;
     } 
  }
  Last_State = clk_State; // Updates the previous state of the clock with the current state
} 

if(menu_activated==3)
   {
  clk_State =   (PINB & B00000001); //pin 8 state? 
  dt_State  =   (PINB & B00000010); 
  if (clk_State != Last_State){     
     // If the data state is different to the clock state, that means the encoder is rotating clockwise
     if (dt_State != clk_State) { 
       ki = ki+1 ;
     }
     else {
       ki = ki-1;
     } 
  }
  Last_State = clk_State; // Updates the previous state of the clock with the current state
}

 if(menu_activated==4)
   {
  clk_State =   (PINB & B00000001); //pin 8 state? 
  dt_State  =   (PINB & B00000010); 
  if (clk_State != Last_State){     
     // If the data state is different to the clock state, that means the encoder is rotating clockwise
     if (dt_State != clk_State) { 
       kd = kd+1 ;
     }
     else {
       kd = kd-1;
     } 
  }
  Last_State = clk_State; // Updates the previous state of the clock with the current state
}

  //Push button was pressed!
  if (PINB & B00001000) //Pin D11 is HIGH?
  {       
    button_pressed = 1;
  } 
  //We navigate through the 4 menus with each button pressed
  else if(button_pressed == 1)
  {
    if(menu_activated==4)
   {
    menu_activated = 0;  
    PID_values_fixed=1;
    button_pressed=0; 
    delay(1000);
   }
   if(menu_activated==3)
   {
    menu_activated = menu_activated + 1;  
    button_pressed=0; 
    kd = kd + 1; 
    delay(1000);
   }
   if(menu_activated==2)
   {
    menu_activated = menu_activated + 1;  
    button_pressed=0; 
    ki = ki + 1; 
    delay(1000);
   }
   if(menu_activated==1)
   {
    menu_activated = menu_activated + 1;  
    button_pressed=0; 
    kp = kp + 1; 
    delay(1000);
   }
   if(menu_activated==0 && PID_values_fixed != 1)
   {
    menu_activated = menu_activated + 1;  
    button_pressed=0;
    set_temperature = set_temperature+1;   
    delay(1000);
   }
   PID_values_fixed = 0;
  }  
}

//triac control
// begin AC interrupt routine
// delay() will not work!
void acon()  
{
  delayMicroseconds(CONT_phase);
  digitalWrite(triacPulse, HIGH);
  delayMicroseconds(200);  
  // delay 200 uSec on output pulse to turn on triac
  digitalWrite(triacPulse, LOW);
} 
