////////////////////////////////////////////
//  
//  HARD:RPi_CON01.ce3
//
//  DATE: 2018.7.12.
//
// PORT
//  AN0(IN ): Vbat
//  GP1(OUT): RPi_Power
//  GP2(OUT): Charge
//  GP3(IN ): Status from RPi
//  GP4(OUT): ShutDown to RPi 
//  GP5(OUT): Monitor LED
//
////////////////////////////////////////////
#include <12f1822.h>
#device ADC=10
#fuses INTRC_IO,NOWDT,PUT,NOMCLR,NOBROWNOUT,NOIESO,NOFCMEN,NOCPD,NOPROTECT,NOCLKOUT
#use delay(CLOCK = 31000)         //NbN31KHz
#use fast_io(A)

#define RPi_Power_ON    output_high(PIN_A1);
#define RPi_Power_OFF   output_low(PIN_A1);
#define MON_ON    output_high(PIN_A5);
#define MON_OFF   output_low(PIN_A5);
#define RPi_ShutDown   output_high(PIN_A4);
#define RPi_ShutDown_Clear output_low(PIN_A4);
#define Charge_ON    output_high(PIN_A2);
#define Charge_OFF   output_low(PIN_A2);
#define BAT      0    //Port for Battery Voltage
#define Rpi_STAT  pin_a3    //Complete send Mail=H/ShutDown=L

//Battery Contorol Voltage
#define  V_kahouden  498   //10.6V (21.78VFull By 10bit)
#define  V_kajyuuden  639   //13.6V (21.78VFull By 10bit)
#define  V_saisetsu  611   //13.0V (21.78VFull By 10bit)

//
#define  Rpi_INTERVAL   1800  //sec

//*** Global ****
unsigned long  sec_cnt,sec_tmp;
long vbat;


#INT_TIMER0           //Timer0 Interrupt
void Interval(void){
   static int charge_flg=0;
   int   mon_stat=0;

   set_timer0(13);      //1sec
   sec_cnt++;
   sec_tmp++;

   if((sec_cnt % 5)==0){MON_ON mon_stat=1;}  //Monitor LED
   setup_adc(ADC_CLOCK_INTERNAL);
   set_adc_channel( BAT );
   delay_cycles(1);      //1(129us)ΰ
   vbat = read_adc();         //vbat=AD
   setup_adc(ADC_OFF);

      //**** ChargeControl ***
      if( vbat> V_saisetsu ){
         if( vbat>V_kajyuuden ){
            Charge_OFF
            charge_flg=1;
         }
         else if(charge_flg==1){
            Charge_OFF
         }
         else{
            Charge_ON
         }
      }
      else{
         Charge_ON
         charge_flg=0;
      }
      //**********************//
      if(mon_stat == 1){MON_OFF mon_stat=0;}
}           //6.5msec


void main(void)
{

   setup_oscillator(OSC_31KHZ);         //clock=31KHz

   //setup IO & A/D
   output_a(0x00);

   setup_vref( VREF_OFF  );
   //setup_adc_ports(sAN0);
   setup_adc(ADC_OFF);
   set_tris_a(0b00001001);
   
   // setup_counters(T0_INTERNAL,T0_DIV_32);  //Not drive
   setup_timer_0(T0_INTERNAL | T0_DIV_32);  //242.2Hz
   sec_cnt=0;
   
   RPi_Power_OFF RPi_ShutDown_Clear Charge_OFF
   
   enable_interrupts(INT_TIMER0);
   enable_interrupts(GLOBAL);   

   sec_cnt= Rpi_INTERVAL;
   while(1){     
      //*** Rpi ***
      if( (vbat>V_kahouden) && (sec_cnt> Rpi_INTERVAL ) ){
         sec_cnt=0;
         RPi_Power_ON MON_ON
         sec_tmp=0;while(sec_tmp<20){;}
         while( input(RPi_STAT)==0 ){;}
         sec_tmp=0;while(sec_tmp<2){;}
         RPi_ShutDown delay_ms(8000); RPi_ShutDown_Clear
         while( input(RPi_STAT)==1 ){;}
         sec_tmp=0;while(sec_tmp<20){;}
         RPi_Power_OFF MON_OFF
      }
      
   }
}



